;**************************************************************************
;* FileName	: TIMER.ASM
;* Date 	: 01.22.2001
;* File version : v1.00
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: TIMER test
;*		  ==================================
;*		  Test for TIMER module.
;*
;*
;*-------------------------------------------------------------------------
;*
;* Connections	:	       _______	 _______
;*			      |       \_/	|
;*		       n.c. ==| RA2	    RA1 |== RS232_RxD
;*		       n.c. ==| RA3	    RA0 |== RS232_TxD
;*		       n.c. ==| RA4	   OSC1 |== quartz1
;*		     -reset ==|-MCLR	   OSC2 |== quartz2
;*			GND ==| Vss	    Vdd |== +5V
;*		      LED_1 ==| RB0	    RB7 |== n.c.
;*		      LED_2 ==| RB1	    RB6 |== n.c.
;*		       n.c. ==| RB2	    RB5 |== n.c.
;*		       n.c. ==| RB3	    RB4 |== n.c.
;*			      |_________________|
;*				   PIC16F84 
;*
;*-------------------------------------------------------------------------
;*
;* Resource	: Ports 	  :    xx /   13 pin   (   %)
;*		  Data memory	  :    xx /   68 byte  (   %)
;*		  Prog memory	  :   xxx / 1024 word  (   %)
;*		  Timer0	  : 
;*		  Interrupt	  : 
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  01/22/2001 	Initial Release
;*
;**************************************************************************

	list	p=16f84, r=dec, l=0

;***** INCLUDES ***********************************************************

	include "p16f84.inc"
	include "..\..\modules\base.inc"

;***** CONFIG ************************************************************

	__CONFIG _XT_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF
	__IDLOCS 0100h			; version

	EXPAND_MACROS	EXP_COND	; macro expansion mode

;***** DEFINES ************************************************************
#define DEBUG				; for some debug message
;#define NO_DELAY			; for fast simulation
;#define NO_RS232			; for fast simulation

xtal_freq	EQU	4 MHz	  	; crystal frequency
core_freq	EQU	xtal_freq/4	; core operating frequency

TIMER_TMR0pre	EQU	64	  	; timer0 prescaler value (2, 4, .. 256)
TIMER_TMR0val	EQU	128		; TMR0 init value (1-256)


; ----- porta -------------------------------------------------------------

#define		RS232_TxD	PORTA,0		; out
#define		RS232_RxD	PORTA,1		; in
#define		PORTA_2		PORTA,2		; free
#define		PORTA_3		PORTA,3		; free
#define		PORTA_4		PORTA,4		; free (open drain!)

; ----- portb -------------------------------------------------------------

#define		LED_1		PORTB,0		; out
#define		LED_2		PORTB,1		; out
#define		PORTB_2		PORTB,2		; free
#define		PORTB_3		PORTB,3		; free
#define		PORTB_4		PORTB,4		; free
#define		PORTB_5		PORTB,5		; free
#define		PORTB_6		PORTB,6		; free
#define		PORTB_7		PORTB,7		; free


;***** MACROS *************************************************************

	include	"..\..\modules\wait.mac"
	include	"..\..\modules\rs232.mac"
	include "..\..\modules\timer.mac"


;***** CONSTANT DEFINITIONS ***********************************************

option_val	EQU	00000010b	; Timer source : intclk / 8
trisa_val	EQU	00011111b	; TRISA value (all input)
trisb_val	EQU	11111100b	; TRISB value
intcon_val	EQU	00100000b	; Interrupt control




;***** VARIABLE DEFINITIONS ***********************************************
	cblock	0Ch
		w_temp			; variable used for context saving
		status_temp		; variable used for context saving
		fsr_temp		; variable used for context saving

		TEMP			; dummy register

		WAIT_TIMER

		TIMER_1
		TIMER_2

		TIMER_3:2		; 16 bit timer

	endc




;***** ENTRY POINT ********************************************************
		ORG	000h		; processor reset vector
		clrwdt
		clrf	PORTA		; clear latch
		clrf	PORTB		; clear latch
		goto	start


;***** IT HANDLER *********************************************************
;* Interrupt handler
;* <comment>
;**************************************************************************
		ORG	004h		; interrupt vector location
		movwf	w_temp		; save off current W register contents
		swapf	STATUS,w	; move STATUS register into W register
		movwf	status_temp	; save off contents of STATUS register

		movf	FSR,w		; save FSR
		movwf	fsr_temp
;--------------------------------------------------------------------------

		bank0		
		TIMER_UPDATE WAIT_TIMER
		TIMER_UPDATE TIMER_1
		TIMER_UPDATE TIMER_2
		TIMER_UPDATE16 TIMER_3

		TIMER_SETTMR0

;--------------------------------------------------------------------------
		movf	fsr_temp,w	; restore FSR
		movwf	FSR

		swapf	status_temp,w	; retrieve copy of STATUS register
		movwf	STATUS		; restore pre-isr STATUS register contents
		swapf	w_temp,f
		swapf	w_temp,w	; restore pre-isr W register contents
		retfie			; return from interrupt




;***** REAL PROGRAM START *************************************************
start
;----- set configuration --------------------------------------------------
		bank1
		ERRORLEVEL -302
		movlw	option_val
		movwf	OPTION_REG	; set OPTION_REG
		movlw	trisa_val
		movwf	TRISA		; set TRISA
		movlw	trisb_val
		movwf	TRISB		; set TRISB

		bank0
		ERRORLEVEL +302
		movlw	intcon_val	
		movwf	INTCON		; set INTCON



;----- other inits --------------------------------------------------------

		RS232_INIT	9600, 8, 1
		TIMER_INIT

		call	LED1_init
		call	LED2_init


		TIMER_WAIT	WAIT_TIMER, 100		; delay 100 mS

		
		TIMER_START16	TIMER_3, 10000
		RS232_SENDCS	"TIMER_3 start!\r\n"

;----- main cycle  --------------------------------------------------------



main_cycle	nop

		TIMER_OUT	TIMER_1, LED1_run
		TIMER_OUT	TIMER_2, LED2_run
		TIMER_OUT16	TIMER_3, main_end

		goto	main_cycle


;--------------------------------------------------------------------------
LED1_init	bcf	LED_1
		
		TIMER_START	TIMER_1, 300
		RS232_SENDCS	"TIMER_1 start!\r\n"

		return

;--------------------------------------------------------------------------
LED2_init	bcf	LED_2

		TIMER_START	TIMER_2, 1000
		RS232_SENDCS	"TIMER_2 start!\r\n"

		return




;--------------------------------------------------------------------------
LED1_run	nop
		RS232_SENDCS	"T1\r\n"
		TIMER_START	TIMER_1, 300

		btfsc	LED_1
		goto	LED1_1
		bsf	LED_1
		goto	main_cycle
				
LED1_1		bcf	LED_1
		goto	main_cycle
		


;--------------------------------------------------------------------------
LED2_run	nop
		RS232_SENDCS	"T2\r\n"
		TIMER_START	TIMER_2, 1000

		btfsc	LED_2
		goto	LED2_1
		bsf	LED_2
		goto	main_cycle
				
LED2_1		bcf	LED_2
		goto	main_cycle


;--------------------------------------------------------------------------
main_end	nop
		RS232_SENDCS	"T3\r\n"
		RS232_SENDCS	"End of Program!\r\n"

main_1		goto	main_1


;***** SUB1 ***************************************************************
;* 
;* Input :              		; 
;* Output:  				; 
;* Clock :   
;**************************************************************************

sub1 


		return

;**************************************************************************

	include	"..\..\modules\rs232.inc"
	include	"..\..\modules\wait.inc"


	end				  ; end of program








